//****************************************************************************
//
// Copyright (c) ALTAP, spol. s r.o. All rights reserved.
//
// This is a part of the Altap Salamander SDK library.
//
// The SDK is provided "AS IS" and without warranty of any kind and 
// ALTAP EXPRESSLY DISCLAIMS ALL WARRANTIES, EXPRESS AND IMPLIED, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE and NON-INFRINGEMENT.
//
//****************************************************************************

#include <windows.h>
#ifdef _MSC_VER
#include <crtdbg.h>
#endif // _MSC_VER
#include <limits.h>
#include <process.h>
#include <commctrl.h>
#include <ostream>

#if defined(_DEBUG) && defined(_MSC_VER)  // without passing file+line to 'new' operator, list of memory leaks shows only 'crtdbg.h(552)'
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

#include "spl_com.h"
#include "spl_base.h"
#include "dbg.h"
#include "auxtools.h"

//
// ****************************************************************************
// CThreadQueue
//

CThreadQueue::CThreadQueue()
{
  Head = NULL;
  Continue = CreateEvent(NULL, FALSE, FALSE, NULL);
}

CThreadQueue::~CThreadQueue()
{
  ClearFinishedThreads();    // neni treba sekce, uz by mel pouzivat jen jeden thread
  if (Continue != NULL) CloseHandle(Continue);
  if (Head != NULL) TRACE_E("Some thread is still running!");  // nemelo by nastat...
}

void
CThreadQueue::ClearFinishedThreads()
{
  CThreadQueueItem *last = NULL;
  CThreadQueueItem *act = Head;
  while (act != NULL)
  {
    DWORD ec;
    if (GetExitCodeThread(act->Thread, &ec) && ec != STILL_ACTIVE)
    {  // tento thread uz skoncil, vyhodime ho ze seznamu
      if (last != NULL) last->Next = act->Next;
      else Head = act->Next;
      CloseHandle(act->Thread);
      delete act;
      act = (last != NULL ? last->Next : Head);
    }
    else
    {
      last = act;
      act = act->Next;
    }
  }
}

BOOL
CThreadQueue::Add(CThreadQueueItem *item)
{
  // nejprve vyhodime thready, ktere se jiz ukoncily
  ClearFinishedThreads();

  // pridame novy thread
  if (item != NULL)
  {
    item->Next = Head;
    Head = item;
    return TRUE;
  }
  return FALSE;
}

BOOL
CThreadQueue::WaitForExit(HANDLE thread, int milliseconds)
{
  CALL_STACK_MESSAGE2("CThreadQueue::WaitForExit(, %d)", milliseconds);
  BOOL ret = TRUE;
  if (thread != NULL)
  {
    CS.Enter();

    CThreadQueueItem *last = NULL;
    CThreadQueueItem *act = Head;   // zkusime najit otevreny handle threadu
    while (act != NULL)
    {
      if (act->Thread == thread) break;
      last = act;
      act = act->Next;
    }
    if (act != NULL)  // nalezeny handle threadu musime vyradit ze seznamu (aby nam ho v jinem threadu nezavreli)
    {
      if (last != NULL) last->Next = act->Next;
      else Head = act->Next;
    }

    CS.Leave();

    if (act != NULL)   // handle threadu nalezeny - muzeme na nej cekat, pak ho zrusime
    {
      ret = WaitForSingleObject(act->Thread, milliseconds) != WAIT_TIMEOUT;
      if (ret)  // thread ukoncen
      {
        CloseHandle(act->Thread);
        delete act;
      }
      else   // thread nebyl ukoncen, pridame ho zpet do fronty
      {
        CS.Enter();

        act->Next = Head;
        Head = act;

        CS.Leave();
      }
    }
  }
  else TRACE_E("CThreadQueue::WaitForExit(): Nothing to wait for (parameter 'thread'==NULL)!");
  return ret;
}

void
CThreadQueue::KillAll()
{
  CS.Enter();

  // vykillujeme vsechny thready, ktere nehodlaji koncit sami
  CThreadQueueItem *last;
  CThreadQueueItem *item = Head;
  while (item != NULL)
  {
    last = item;
    item = item->Next;
    DWORD ec;
    if (!GetExitCodeThread(last->Thread, &ec) || ec == STILL_ACTIVE)
    {   // thread jeste nejspis bezi, zabijeme ho
      TRACE_E("Thread has not ended itself, we must terminate it.");
      TerminateThread(last->Thread, 666);
      WaitForSingleObject(last->Thread, INFINITE);
    }
    CloseHandle(last->Thread);
    delete last;
  }
  Head = NULL;

  CS.Leave();
}

struct CThreadBaseData
{
  unsigned (WINAPI *Body)(void *);
  void *Param;
  HANDLE Continue;
};

unsigned __stdcall
CThreadQueue::ThreadBase(void *param)
{
  CThreadBaseData *d = (CThreadBaseData *)param;

  // zaloha dat na stack ('d' prestane byt platne po 'Continue')
  unsigned (WINAPI *threadBody)(void *) = d->Body;
  void *threadParam = d->Param;

  SetEvent(d->Continue);   // pustime dal hl. thread
  d = NULL;

  // spustime nas thread
  return SalamanderDebug->CallWithCallStack(threadBody, threadParam);
}

HANDLE
CThreadQueue::StartThread(unsigned (WINAPI *body)(void *), void *param, unsigned stack_size,
                          HANDLE *threadHandle, BOOL duplicateThreadHandle)
{
  if (Continue == NULL)
  {
    TRACE_E("Unable to start thread, because Continue event was not created.");
    if (threadHandle != NULL) *threadHandle = NULL;
    return NULL;
  }

  CS.Enter();

  CThreadBaseData data;
  data.Body = body;
  data.Param = param;
  data.Continue = Continue;

  // viewer spustime v novem threadu
  unsigned threadID;
  HANDLE thread = (HANDLE)_beginthreadex(NULL, stack_size, CThreadQueue::ThreadBase, &data,
                                         CREATE_SUSPENDED, &threadID);
  if (thread == NULL)
  {
    TRACE_E("Unable to start thread.");
    if (threadHandle != NULL) *threadHandle = NULL;

    CS.Leave();

    return NULL;
  }
  else
  {
    // je-li treba, duplikujeme handle threadu
    HANDLE dupHandle = thread;
    if (duplicateThreadHandle)
    {
      if (DuplicateHandle(GetCurrentProcess(), thread, GetCurrentProcess(), &dupHandle,
                          0, FALSE, DUPLICATE_SAME_ACCESS) == 0)
      {
        TRACE_E("Unable to duplicate thread handle.");
        TerminateThread(thread, 666);           // je suspended, takze nebude v zadne kriticke sekci, atd.
        WaitForSingleObject(thread, INFINITE);  // pockame, az skonci
        CloseHandle(thread);
        if (threadHandle != NULL) *threadHandle = NULL;

        CS.Leave();

        return NULL;
      }
    }
    // pridame thread do fronty threadu tohoto pluginu
    if (!Add(new CThreadQueueItem(thread)))
    {
      TRACE_E("Unable to add thread to the queue.");
      TerminateThread(thread, 666);           // je suspended, takze nebude v zadne kriticke sekci, atd.
      WaitForSingleObject(thread, INFINITE);  // pockame, az skonci
      CloseHandle(thread);
      if (duplicateThreadHandle) CloseHandle(dupHandle);
      if (threadHandle != NULL) *threadHandle = NULL;

      CS.Leave();

      return NULL;
    }

    if (threadHandle != NULL) *threadHandle = dupHandle;
    SalamanderDebug->TraceAttachThread(thread, threadID);
    ResumeThread(thread);

    WaitForSingleObject(Continue, INFINITE);   // pockame na predani dat do CThreadQueue::ThreadBase

    CS.Leave();

    return dupHandle;
  }
}

//
// ****************************************************************************
// CThread
//

CThread::CThread(const char *name)
{
  if (name != NULL) lstrcpyn(Name, name, 101);
  else Name[0] = 0;
  Thread = NULL;
}

unsigned WINAPI
CThread::UniversalBody(void *param)
{
  CThread *thread = (CThread *)param;
  CALL_STACK_MESSAGE2("CThread::UniversalBody(thread name = \"%s\")", thread->Name);
  if (SalamanderVersion >= 25)
    SalamanderDebug->SetThreadNameInVCAndTrace_P(thread->Name);
  else
    SalamanderDebug->TraceSetThreadName(thread->Name);

  unsigned ret = thread->Body();  // spusteni tela threadu

  delete thread;  // likvidace objektu threadu
  return ret;
}

HANDLE
CThread::Create(CThreadQueue &queue, unsigned stack_size, BOOL duplicateThreadHandle)
{
  return queue.StartThread(UniversalBody, this, stack_size, &Thread, duplicateThreadHandle);
}
