//****************************************************************************
//
// Copyright (c) ALTAP, spol. s r.o. All rights reserved.
//
// This is a part of the Altap Salamander SDK library.
//
// The SDK is provided "AS IS" and without warranty of any kind and 
// ALTAP EXPRESSLY DISCLAIMS ALL WARRANTIES, EXPRESS AND IMPLIED, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE and NON-INFRINGEMENT.
//
//****************************************************************************

#ifndef __AUXTOOLS_H
#define __AUXTOOLS_H

//
// ****************************************************************************
// CThreadQueue
//

struct CThreadQueueItem
{
  HANDLE Thread;
  CThreadQueueItem *Next;

  CThreadQueueItem(HANDLE thread) {Thread = thread; Next = NULL;}
};

class CThreadQueue
{
  protected:
    CThreadQueueItem *Head;
    HANDLE Continue;  // musime pockat na predani dat do startovaneho threadu

    struct CCS  // pristup z vice threadu -> nutna synchronizace
    {
      CRITICAL_SECTION cs;

      CCS() {InitializeCriticalSection(&cs);}
      ~CCS() {DeleteCriticalSection(&cs);}

      void Enter() {EnterCriticalSection(&cs);}
      void Leave() {LeaveCriticalSection(&cs);}
    } CS;

  public:
    CThreadQueue();
    ~CThreadQueue();

    // spusti funkci 'body' s parametrem 'param' v nove vytvorenem threadu se zasobnikem
    // o velikosti 'stack_size' (prip. automaticky roste); vraci handle threadu nebo jeho
    // duplikat (je-li 'duplicateThreadHandle' TRUE) nebo NULL pri chybe; zavreni handlu
    // threadu zajistuje tento objekt fronty (o duplikat se ovsem musi postarat volajici);
    // neni-li 'threadHandle' NULL, vraci se v nem navratova hodnota metody
    // POZOR: -thread se muze spustit se zpozdenim az po navratu ze StartThread()
    //         (je-li 'param' ukazatel na strukturu ulozenou na zasobniku, je nutne
    //          synchronizovat predani dat z 'param' - hl. thread musi pockat
    //          na prevzeti dat novym threadem)
    //        -thread muze dobehnout jeste pred navratem ze StartThread()
    //        -vraceny handle threadu jiz muze byt zavreny pokud thread dobehne pred
    //         navratem ze StartThread() (jen pokud je 'duplicateThreadHandle' FALSE)
    // mozne volat z libovolneho threadu
    HANDLE StartThread(unsigned (WINAPI *body)(void *), void *param, unsigned stack_size,
                       HANDLE *threadHandle, BOOL duplicateThreadHandle = FALSE);

    // ceka na ukonceni threadu z teto fronty; 'thread' je handle threadu, ktery jiz muze
    // byt i zavreny (zavira tento objekt pri volani StartThread a KillAll); pokud se docka
    // ukonceni threadu, vyradi thread z fronty a zavre jeho handle
    BOOL WaitForExit(HANDLE thread, int milliseconds = INFINITE);

    // zabije vsechny thready, ktere jeste neskoncily
    // mozne volat z libovolneho threadu
    void KillAll();

  protected:   // vnitrni nesynchronizovane metody
    BOOL Add(CThreadQueueItem *item);  // prida polozku do fronty, vraci uspech
    void ClearFinishedThreads();  // vyradi z fronty thready, ktere jiz dobehly
    static unsigned __stdcall CThreadQueue::ThreadBase(void *param);  // univerzalni body threadu
};

//
// ****************************************************************************
// CThread
//
// POZOR: musi se alokovat (neni mozne mit CThread jen na stacku); dealokuje se sam
//        jen v pripade uspesneho vytvoreni threadu metodou Create()

class CThread
{
  public:
    HANDLE Thread;   // handle threadu (NULL = thread jeste nebezi/nebezel)

  protected:
    char Name[101];  // buffer pro jmeno threadu (pouziva se v TRACE a CALL-STACK pro identifikaci threadu)
    // POZOR: pokud budou data threadu obsahovat odkazy na stack nebo jine docasne objekty,
    //        je potreba zajistit, aby se s temito odkazy pracovalo jen po dobu jejich platnosti

  public:
    CThread(const char *name = NULL);
    virtual ~CThread() {} // aby se spravne volaly destruktory potomku

    // vytvoreni (start) threadu ve fronte threadu 'queue'; 'stack_size' je velikost
    // zasobniku noveho threadu v bytech; vraci handle noveho threadu nebo jeho duplikat (je-li
    // 'duplicateThreadHandle' TRUE) nebo NULL pri chybe; zavreni handlu zajistuje objekt 'queue'
    // (o duplikat se ovsem musi postarat volajici); pokud se podari vytvorit thread, je tento
    // objekt dealokovan pri ukonceni threadu, pri chybe spousteni dealokace objektu neprobehne
    // POZOR: bez pridani synchronizace muze thread dobehnout jeste pred navratem z Create() -> ukazatel
    //        "this" je proto po uspesnem volani Create() nutne povazovat za neplatny, to
    //        same plati pro vraceny handle threadu (pokud je 'duplicateThreadHandle' FALSE)
    // mozne volat z libovolneho threadu
    HANDLE Create(CThreadQueue &queue, unsigned stack_size = 0, BOOL duplicateThreadHandle = FALSE);

    // vraci handle threadu (NULL = jeste nebezi/nebezel)
    HANDLE GetHandle() {return Thread;}

    // vraci jmeno threadu
    const char *GetName() {return Name;}

    // tato metoda obsahuje telo threadu
    virtual unsigned Body() = 0;

  protected:
    static unsigned WINAPI UniversalBody(void *param);    // pomocna metoda pro spousteni threadu
};

#endif // __AUXTOOLS_H
